@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2006-2016,2018,2021-2022 Con Kolivas
+   Copyright (C) 2006-2016,2018,2021-2022,2026 Con Kolivas
    Copyright (C) 2011 Peter Hyman
    Copyright (C) 1998-2003 Andrew Tridgell
 
@@ -56,6 +56,7 @@
 #include "stream.h"
 
 #define MAGIC_LEN (24)
+#define STDIO_TMPFILE_BUFFER_SIZE (65536) // used in read_tmpinfile and dump_tmpoutfile
 
 static void release_hashes(rzip_control *control);
 
@@ -65,7 +66,7 @@ static i64 fdout_seekto(rzip_control *control, i64 pos)
 		pos -= control->out_relofs;
 		control->out_ofs = pos;
 		if (unlikely(pos > control->out_len || pos < 0)) {
-			print_err("Trying to seek to %lld outside tmp outbuf in fdout_seekto\n", pos);
+			print_err("Trying to seek to %"PRId64" outside tmp outbuf in fdout_seekto\n", pos);
 			return -1;
 		}
 		return 0;
@@ -90,6 +91,32 @@ i64 get_ram(rzip_control *control)
 
 	return ramsize;
 }
+#elif defined(__OpenBSD__)
+# include <sys/resource.h>
+i64 get_ram(rzip_control *control)
+{
+	struct rlimit rl;
+	i64 ramsize = (i64)sysconf(_SC_PHYS_PAGES) * PAGE_SIZE;
+
+	/* Raise limits all the way to the max */
+
+	if (getrlimit(RLIMIT_DATA, &rl) == -1)
+		fatal_return(("Failed to get limits in get_ram\n"), -1);
+
+	rl.rlim_cur = rl.rlim_max;
+	if (setrlimit(RLIMIT_DATA, &rl) == -1)
+		fatal_return(("Failed to set limits in get_ram\n"), -1);
+
+	/* Declare detected RAM to be either the max RAM available from
+	physical memory or the max RAM allowed by RLIMIT_DATA, whatever
+	is smaller, to prevent the heuristics from selecting
+	compression windows which cause lrzip to go into deep swap */
+
+	if (rl.rlim_max < ramsize)
+		return rl.rlim_max;
+
+	return ramsize;
+}
 #else /* __APPLE__ */
 i64 get_ram(rzip_control *control)
 {
@@ -202,10 +229,14 @@ static bool get_magic(rzip_control *control, char *magic)
 
 	/* Support the convoluted way we described size in versions < 0.40 */
 	if (control->major_version == 0 && control->minor_version < 4) {
+		i64 ormask;
 		memcpy(&v, &magic[6], 4);
 		expected_size = ntohl(v);
 		memcpy(&v, &magic[10], 4);
-		expected_size |= ((i64)ntohl(v)) << 32;
+		ormask = ((i64)ntohl(v));
+		if (ormask >  0x7FFFFFFF)
+			failure_return(("Invalid expected size encoded in magic header\n"), false);
+		expected_size |= ormask << 32;
 	} else {
 		memcpy(&expected_size, &magic[6], 8);
 		expected_size = le64toh(expected_size);
@@ -243,7 +274,7 @@ static bool get_magic(rzip_control *control, char *magic)
 		memcpy(&control->salt, &magic[6], 8);
 		control->st_size = expected_size = 0;
 		control->encloops = enc_loops(control->salt[0], control->salt[1]);
-		print_maxverbose("Encryption hash loops %lld\n", control->encloops);
+		print_maxverbose("Encryption hash loops %"PRId64"\n", control->encloops);
 	} else if (ENCRYPT) {
 		print_output("Asked to decrypt a non-encrypted archive. Bypassing decryption.\n");
 		control->flags &= ~FLAG_ENCRYPT;
@@ -386,7 +417,7 @@ static bool flush_tmpoutbuf(rzip_control *control)
 /* Dump temporary outputfile to perform stdout */
 static bool dump_tmpoutfile(rzip_control *control)
 {
-	int tmpchar, fd_out = control->fd_out;
+	int fd_out = control->fd_out;
 	FILE *tmpoutfp;
 
 	if (unlikely(fd_out == -1))
@@ -399,9 +430,35 @@ static bool dump_tmpoutfile(rzip_control *control)
 	rewind(tmpoutfp);
 
 	if (!TEST_ONLY) {
+		char* buf;
+
 		print_verbose("Dumping temporary file to control->outFILE.\n");
-		while ((tmpchar = fgetc(tmpoutfp)) != EOF)
-			putchar(tmpchar);
+		fflush(control->outFILE);
+
+		buf = malloc(STDIO_TMPFILE_BUFFER_SIZE);
+		if (unlikely(!buf))
+			fatal_return(("Failed to allocate buffer in dump_tmpoutfile\n"), false);
+
+		while (1) {
+			ssize_t num_read, num_written;
+			num_read = fread(buf, 1, STDIO_TMPFILE_BUFFER_SIZE, tmpoutfp);
+			if (unlikely(num_read == 0)) {
+				if (ferror(tmpoutfp)) {
+					dealloc(buf);
+					fatal_return(("Failed read in dump_tmpoutfile\n"), false);
+				} else {
+					break; // must be at EOF
+				}
+			}
+
+			num_written = fwrite(buf, 1, num_read, control->outFILE);
+			if (unlikely(num_written != num_read)) {
+				dealloc(buf);
+				fatal_return(("Failed write in dump_tmpoutfile\n"), false);
+			}
+		}
+
+		dealloc(buf);
 		fflush(control->outFILE);
 		rewind(tmpoutfp);
 	}
@@ -512,7 +569,7 @@ static bool read_tmpinmagic(rzip_control *control)
 bool read_tmpinfile(rzip_control *control, int fd_in)
 {
 	FILE *tmpinfp;
-	int tmpchar;
+	char* buf;
 
 	if (fd_in == -1)
 		return false;
@@ -522,9 +579,30 @@ bool read_tmpinfile(rzip_control *control, int fd_in)
 	if (unlikely(tmpinfp == NULL))
 		fatal_return(("Failed to fdopen in tmpfile\n"), false);
 
-	while ((tmpchar = getchar()) != EOF)
-		fputc(tmpchar, tmpinfp);
+	buf = malloc(STDIO_TMPFILE_BUFFER_SIZE);
+	if (unlikely(!buf))
+		fatal_return(("Failed to allocate buffer in read_tmpinfile\n"), false);
 
+	while (1) {
+		ssize_t num_read, num_written;
+		num_read = fread(buf, 1, STDIO_TMPFILE_BUFFER_SIZE, stdin);
+		if (unlikely(num_read == 0)) {
+			if (ferror(stdin)) {
+				dealloc(buf);
+				fatal_return(("Failed read in read_tmpinfile\n"), false);
+			} else {
+				break; // must be at EOF
+			}
+		}
+
+		num_written = fwrite(buf, 1, num_read, tmpinfp);
+		if (unlikely(num_written != num_read)) {
+			dealloc(buf);
+			fatal_return(("Failed write in read_tmpinfile\n"), false);
+		}
+	}
+
+	dealloc(buf);
 	fflush(tmpinfp);
 	rewind(tmpinfp);
 	return true;
@@ -704,6 +782,12 @@ static void release_hashes(rzip_control *control)
 
 static void clear_rulist(rzip_control *control)
 {
+	/* If we're unable to safely clean up thread-related memory due to
+	 * a failure in decompression, let the small memory leak be cleaned
+	 * up by process exit */
+	if (unlikely(control->thread_count > 0)) {
+		return;
+	}
 	while (control->ruhead) {
 		struct runzip_node *node = control->ruhead;
 		struct stream_info *sinfo = node->sinfo;
@@ -783,7 +867,7 @@ bool decompress_file(rzip_control *control)
 	else if (STDIN) {
 		fd_in = open_tmpinfile(control);
 		read_tmpinmagic(control);
-		if (ENCRYPT)
+		if (ENCRYPT && !control->passphrase)
 			failure_return(("Cannot decompress encrypted file from STDIN\n"), false);
 		expected_size = control->st_size;
 		if (unlikely(!open_tmpinbuf(control)))
@@ -840,7 +924,7 @@ bool decompress_file(rzip_control *control)
 		if (unlikely(!read_magic(control, fd_in, &expected_size)))
 			return false;
 		if (unlikely(expected_size < 0))
-			fatal_return(("Invalid expected size %lld\n", expected_size), false);
+			fatal_return(("Invalid expected size %"PRId64"\n", expected_size), false);
 	}
 
 	if (!STDOUT && !TEST_ONLY) {
@@ -889,7 +973,7 @@ bool decompress_file(rzip_control *control)
 	if (!expected_size)
 		expected_size = control->st_size;
 	if (!ENCRYPT)
-		print_output("[OK] - %lld bytes                                \n", expected_size);
+		print_output("[OK] - %"PRId64" bytes                                \n", expected_size);
 	else
 		print_output("[OK]                                             \n");
 
@@ -1042,7 +1126,7 @@ bool get_fileinfo(rzip_control *control)
 				fatal_goto(("Failed to read chunk_size in get_fileinfo\n"), error);
 			chunk_size = le64toh(chunk_size);
 			if (unlikely(chunk_size < 0))
-				fatal_goto(("Invalid chunk size %lld\n", chunk_size), error);
+				fatal_goto(("Invalid chunk size %"PRId64"\n", chunk_size), error);
 		}
 	}
 
@@ -1093,7 +1177,7 @@ next_chunk:
 		do {
 			i64 head_off;
 
-			if (unlikely(last_head && last_head < second_last))
+			if (unlikely(last_head && last_head <= second_last))
 				failure_goto(("Invalid earlier last_head position, corrupt archive.\n"), error);
 			second_last = last_head;
 			if (unlikely(last_head + ofs > infile_size))
@@ -1154,7 +1238,7 @@ next_chunk:
 				fatal_goto(("Failed to read chunk_size in get_fileinfo\n"), error);
 			chunk_size = le64toh(chunk_size);
 			if (unlikely(chunk_size < 0))
-				fatal_goto(("Invalid chunk size %lld\n", chunk_size), error);
+				fatal_goto(("Invalid chunk size %"PRId64"\n", chunk_size), error);
 			ofs += 1 + chunk_byte;
 			header_length = 1 + (chunk_byte * 3);
 		}
@@ -1386,6 +1470,27 @@ error:
 	return false;
 }
 
+static int get_available_cpus(void)
+{
+	long sys;
+#ifdef __linux__
+	cpu_set_t mask;
+	CPU_ZERO(&mask);
+
+	if (sched_getaffinity(0, sizeof(mask), &mask) == 0) {
+		int count = CPU_COUNT(&mask);
+		if (count > 0)
+		    return count;
+	}
+#endif
+	/* Fallback to system-wide online CPUs */
+	sys = sysconf(_SC_NPROCESSORS_ONLN);
+	if (sys > 0)
+		return (int)sys;
+
+	return 1;  /* Absolute minimum */
+}
+
 bool initialise_control(rzip_control *control)
 {
 	time_t now_t, tdiff;
@@ -1403,7 +1508,7 @@ bool initialise_control(rzip_control *control)
 	if (unlikely(control->ramsize == -1))
 		return false;
 	/* for testing single CPU */
-	control->threads = PROCESSORS;	/* get CPUs for LZMA */
+	control->threads = get_available_cpus();	/* get CPUs for LZMA */
 	control->page_size = PAGE_SIZE;
 	control->nice_val = 19;
 
