@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2006-2016,2021-2022 Con Kolivas
+   Copyright (C) 2006-2016,2021-2022,2026 Con Kolivas
    Copyright (C) 2011 Peter Hyman
    Copyright (C) 1998-2003 Andrew Tridgell
 
@@ -52,6 +52,7 @@
 #include <dirent.h>
 #include <getopt.h>
 #include <libgen.h>
+#include <inttypes.h>
 
 #include "rzip.h"
 #include "lrzip_core.h"
@@ -162,9 +163,9 @@ static void show_summary(void)
 		if (!TEST_ONLY)
 			print_verbose("The following options are in effect for this %s.\n",
 				      DECOMPRESS ? "DECOMPRESSION" : "COMPRESSION");
-		print_verbose("Threading is %s. Number of CPUs detected: %d\n", control->threads > 1? "ENABLED" : "DISABLED",
+		print_verbose("Threading is %s. Number of CPUs available detected: %d\n", control->threads > 1? "ENABLED" : "DISABLED",
 			      control->threads);
-		print_verbose("Detected %lld bytes ram\n", control->ramsize);
+		print_verbose("Detected %"PRId64" bytes ram\n", control->ramsize);
 		print_verbose("Compression level %d\n", control->compression_level);
 		print_verbose("Nice Value: %d\n", control->nice_val);
 		print_verbose("Show Progress\n");
@@ -199,7 +200,7 @@ static void show_summary(void)
 			else if (NO_COMPRESS)
 				print_verbose("RZIP pre-processing only\n");
 			if (control->window)
-				print_verbose("Compression Window: %lld = %lldMB\n", control->window, control->window * 100ull);
+				print_verbose("Compression Window: %"PRId64" = %lldMB\n", control->window, control->window * 100ull);
 			/* show heuristically computed window size */
 			if (!control->window && !UNLIMITED) {
 				i64 temp_chunk, temp_window;
@@ -209,15 +210,15 @@ static void show_summary(void)
 				else
 					temp_chunk = control->ramsize * 2 / 3;
 				temp_window = temp_chunk / (100 * 1024 * 1024);
-				print_verbose("Heuristically Computed Compression Window: %lld = %lldMB\n", temp_window, temp_window * 100ull);
+				print_verbose("Heuristically Computed Compression Window: %"PRId64" = %lldMB\n", temp_window, temp_window * 100ull);
 			}
 			if (UNLIMITED)
 				print_verbose("Using Unlimited Window size\n");
 		}
 		if (!DECOMPRESS && !TEST_ONLY)
-			print_maxverbose("Storage time in seconds %lld\n", control->secs);
+			print_maxverbose("Storage time in seconds %"PRId64"\n", control->secs);
 		if (ENCRYPT)
-			print_maxverbose("Encryption hash loops %lld\n", control->encloops);
+			print_maxverbose("Encryption hash loops %"PRId64"\n", control->encloops);
 	}
 }
 
@@ -437,6 +438,8 @@ int main(int argc, char *argv[])
 				license();
 				exit(0);
 			}
+			if (!optarg)
+				failure("No level specified with -L\n");
 			control->compression_level = strtol(optarg, &endptr, 10);
 			if (control->compression_level < 1 || control->compression_level > 9)
 				failure("Invalid compression level (must be 1-9)\n");
@@ -571,6 +574,9 @@ int main(int argc, char *argv[])
 			failure("Cannot specify output filename with recursive\n");
 	}
 
+	if (INFO && !SHOW_OUTPUT)
+		failure("Cannot show info and have no output.\n");
+
 	if (VERBOSE && !SHOW_PROGRESS) {
 		print_err("Cannot have -v and -q options. -v wins.\n");
 		control->flags |= FLAG_SHOW_PROGRESS;
@@ -712,7 +718,7 @@ recursion:
 
 		gettimeofday(&start_time, NULL);
 
-		if (unlikely((STDIN || STDOUT) && ENCRYPT))
+		if (!control->passphrase && (unlikely((STDIN || STDOUT) && ENCRYPT)))
 			failure("Unable to work from STDIO while reading password\n");
 
 		memcpy(&local_control, &base_control, sizeof(rzip_control));
