@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2006-2016,2018,2021 Con Kolivas
+   Copyright (C) 2006-2016,2018,2021-2022,2026 Con Kolivas
    Copyright (C) 1998-2003 Andrew Tridgell
 
    This program is free software; you can redistribute it and/or modify
@@ -40,6 +40,8 @@
 # include <arpa/inet.h>
 #endif
 
+#include <inttypes.h>
+
 #include "md5.h"
 #include "runzip.h"
 #include "stream.h"
@@ -97,7 +99,7 @@ static i64 seekto_fdhist(rzip_control *control, i64 pos)
 	if (control->hist_ofs > control->out_len)
 		control->out_len = control->hist_ofs;
 	if (unlikely(control->hist_ofs < 0 || control->hist_ofs > control->out_maxlen)) {
-		print_err("Trying to seek outside tmpoutbuf to %lld in seekto_fdhist\n", control->hist_ofs);
+		print_err("Trying to seek outside tmpoutbuf to %"PRId64" in seekto_fdhist\n", control->hist_ofs);
 		return -1;
 	}
 	return pos;
@@ -115,7 +117,7 @@ static i64 seekto_fdin(rzip_control *control, i64 pos)
 	if (!TMP_INBUF)
 		return lseek(control->fd_in, pos, SEEK_SET);
 	if (unlikely(pos > control->in_len || pos < 0)) {
-		print_err("Trying to seek outside tmpinbuf to %lld in seekto_fdin\n", pos);
+		print_err("Trying to seek outside tmpinbuf to %"PRId64" in seekto_fdin\n", pos);
 		return -1;
 	}
 	control->in_ofs = pos;
@@ -153,11 +155,11 @@ static i64 unzip_literal(rzip_control *control, void *ss, i64 len, uint32 *cksum
 	uchar *buf;
 
 	if (unlikely(len < 0))
-		failure_return(("len %lld is negative in unzip_literal!\n",len), -1);
+		failure_return(("len %"PRId64" is negative in unzip_literal!\n",len), -1);
 
 	buf = (uchar *)malloc(len);
 	if (unlikely(!buf))
-		fatal_return(("Failed to malloc literal buffer of size %lld\n", len), -1);
+		fatal_return(("Failed to malloc literal buffer of size %"PRId64"\n", len), -1);
 
 	stream_read = read_stream(control, ss, 1, buf, len);
 	if (unlikely(stream_read == -1 )) {
@@ -167,7 +169,7 @@ static i64 unzip_literal(rzip_control *control, void *ss, i64 len, uint32 *cksum
 
 	if (unlikely(write_1g(control, buf, (size_t)stream_read) != (ssize_t)stream_read)) {
 		dealloc(buf);
-		fatal_return(("Failed to write literal buffer of size %lld\n", stream_read), -1);
+		fatal_return(("Failed to write literal buffer of size %"PRId64"\n", stream_read), -1);
 	}
 
 	if (!HAS_MD5)
@@ -194,10 +196,10 @@ static i64 read_fdhist(rzip_control *control, void *buf, i64 len)
 static i64 unzip_match(rzip_control *control, void *ss, i64 len, uint32 *cksum, int chunk_bytes)
 {
 	i64 offset, n, total, cur_pos;
-	uchar *buf, *off_buf;
+	uchar *buf;
 
 	if (unlikely(len < 0))
-		failure_return(("len %lld is negative in unzip_match!\n",len), -1);
+		failure_return(("len %"PRId64" is negative in unzip_match!\n",len), -1);
 
 	total = 0;
 	cur_pos = seekcur_fdout(control);
@@ -209,35 +211,38 @@ static i64 unzip_match(rzip_control *control, void *ss, i64 len, uint32 *cksum,
 	if (unlikely(offset == -1))
 		return -1;
 	if (unlikely(seekto_fdhist(control, cur_pos - offset) == -1))
-		fatal_return(("Seek failed by %d from %d on history file in unzip_match\n",
+		fatal_return(("Seek failed by %"PRId64" from %"PRId64" on history file in unzip_match\n",
 		      offset, cur_pos), -1);
 
-	buf = (uchar *)malloc(len);
+	n = MIN(len, offset);
+	if (unlikely(n < 1))
+		fatal_return(("Failed fd history in unzip_match due to corrupt archive\n"), -1);
+
+	buf = (uchar *)malloc(n);
 	if (unlikely(!buf))
-		fatal_return(("Failed to malloc match buffer of size %lld\n", len), -1);
-	off_buf = buf;
+		fatal_return(("Failed to malloc match buffer of size %"PRId64"\n", len), -1);
+
+	if (unlikely(read_fdhist(control, buf, (size_t)n) != (ssize_t)n)) {
+		dealloc(buf);
+		fatal_return(("Failed to read %"PRId64" bytes in unzip_match\n", n), -1);
+	}
 
 	while (len) {
 		n = MIN(len, offset);
 		if (unlikely(n < 1))
 			fatal_return(("Failed fd history in unzip_match due to corrupt archive\n"), -1);
 
-		if (unlikely(read_fdhist(control, off_buf, (size_t)n) != (ssize_t)n)) {
+		if (unlikely(write_1g(control, buf, (size_t)n) != (ssize_t)n)) {
 			dealloc(buf);
-			fatal_return(("Failed to read %d bytes in unzip_match\n", n), -1);
-		}
-		if (unlikely(write_1g(control, off_buf, (size_t)n) != (ssize_t)n)) {
-			dealloc(buf);
-			fatal_return(("Failed to write %d bytes in unzip_match\n", n), -1);
+			fatal_return(("Failed to write %"PRId64" bytes in unzip_match\n", n), -1);
 		}
 
 		if (!HAS_MD5)
-			*cksum = CrcUpdate(*cksum, off_buf, n);
+			*cksum = CrcUpdate(*cksum, buf, n);
 		if (!NO_MD5)
-			md5_process_bytes(off_buf, n, &control->ctx);
+			md5_process_bytes(buf, n, &control->ctx);
 
 		len -= n;
-		off_buf += n;
 		total += n;
 	}
 
@@ -246,21 +251,6 @@ static i64 unzip_match(rzip_control *control, void *ss, i64 len, uint32 *cksum,
 	return total;
 }
 
-static void clear_rulist(rzip_control *control)
-{
-	while (control->ruhead) {
-		struct runzip_node *node = control->ruhead;
-		struct stream_info *sinfo = node->sinfo;
-
-		dealloc(sinfo->ucthreads);
-		dealloc(node->pthreads);
-		dealloc(sinfo->s);
-		dealloc(sinfo);
-		control->ruhead = node->prev;
-		dealloc(node);
-	}
-}
-
 /* decompress a section of an open file. Call fatal_return(() on error
    return the number of bytes that have been retrieved
  */
@@ -300,7 +290,7 @@ static i64 runzip_chunk(rzip_control *control, int fd_in, i64 expected_size, i64
 	else if (control->major_version == 0 && control->minor_version == 4)
 		chunk_bytes = 8;
 	else {
-		print_maxverbose("Reading chunk_bytes at %lld\n", get_readseek(control, fd_in));
+		print_maxverbose("Reading chunk_bytes at %"PRId64"\n", get_readseek(control, fd_in));
 		/* Read in the stored chunk byte width from the file */
 		if (unlikely(read_1g(control, fd_in, &chunk_bytes, 1) != 1))
 			fatal_return(("Failed to read chunk_bytes size in runzip_chunk\n"), -1);
@@ -308,7 +298,7 @@ static i64 runzip_chunk(rzip_control *control, int fd_in, i64 expected_size, i64
 			failure_return(("chunk_bytes %d is invalid in runzip_chunk\n", chunk_bytes), -1);
 	}
 	if (!tally && expected_size)
-		print_maxverbose("Expected size: %lld\n", expected_size);
+		print_maxverbose("Expected size: %"PRId64"\n", expected_size);
 	print_maxverbose("Chunk byte width: %d\n", chunk_bytes);
 
 	ofs = seekcur_fdin(control);
@@ -378,17 +368,13 @@ static i64 runzip_chunk(rzip_control *control, int fd_in, i64 expected_size, i64
 	if (unlikely(close_stream_in(control, ss)))
 		fatal("Failed to close stream!\n");
 
-	/* We can now safely delete sinfo and pthread data of all threads
-	 * created. */
-	clear_rulist(control);
-
 	return total;
 }
 
 /* Decompress an open file. Call fatal_return(() on error
    return the number of bytes that have been retrieved
  */
-i64 runzip_fd(rzip_control *control, int fd_in, int fd_out, int fd_hist, i64 expected_size)
+i64 runzip_fd(rzip_control *control, int fd_in, int fd_hist, i64 expected_size)
 {
 	uchar md5_stored[MD5_DIGEST_SIZE];
 	struct timeval start,end;
@@ -408,17 +394,11 @@ i64 runzip_fd(rzip_control *control, int fd_in, int fd_out, int fd_hist, i64 exp
 			}
 		}
 		total += u;
-		if (TMP_OUTBUF) {
-			if (unlikely(!flush_tmpoutbuf(control))) {
-				print_err("Failed to flush_tmpoutbuf in runzip_fd\n");
+		if (unlikely(!flush_tmpout(control))) {
+			print_err("Failed to flush_tmpout in runzip_fd\n");
 				return -1;
-			}
-		} else if (STDOUT) {
-			if (unlikely(!dump_tmpoutfile(control, fd_out))) {
-				print_err("Failed to dump_tmpoutfile in runzip_fd\n");
-				return -1;
-			}
 		}
+
 		if (TMP_INBUF)
 			clear_tmpinbuf(control);
 		else if (STDIN && !DECOMPRESS) {
@@ -434,7 +414,7 @@ i64 runzip_fd(rzip_control *control, int fd_in, int fd_out, int fd_hist, i64 exp
 		tdiff = end.tv_sec - start.tv_sec;
 		if (!tdiff)
 			tdiff = 1;
-		print_progress("\nAverage DeCompression Speed: %6.3fMB/s\n",
+		print_output("\nAverage DeCompression Speed: %6.3fMB/s\n",
 			       (total / 1024 / 1024) / tdiff);
 	}
 
